@@ -51,8 +51,8 @@ py_an_new(PyTypeObject *type, PyObject *args, PyObject *kwargs) {
   }
   const char *metric = NULL;
 
-  static char * kwlist[] = {"f", "metric", NULL};
-  if (!PyArg_ParseTupleAndKeywords(args, kwargs, "i|s", kwlist, &self->f, &metric))
+  static char const * kwlist[] = {"f", "metric", NULL};
+  if (!PyArg_ParseTupleAndKeywords(args, kwargs, "i|s", (char**)kwlist, &self->f, &metric))
     return NULL;
   if (!metric || !strcmp(metric, "angular")) {
    self->ptr = new AnnoyIndex<int32_t, float, Angular, Kiss64Random>(self->f);
@@ -99,10 +99,10 @@ py_an_load(py_annoy *self, PyObject *args, PyObject *kwargs) {
   char* filename;
   bool res = false;
   if (!self->ptr) 
-    Py_RETURN_NONE;
-  static char * kwlist[] = {"fn", NULL};
-  if (!PyArg_ParseTupleAndKeywords(args, kwargs, "s", kwlist, &filename))
-    Py_RETURN_NONE;
+    return NULL;
+  static char const * kwlist[] = {"fn", NULL};
+  if (!PyArg_ParseTupleAndKeywords(args, kwargs, "s", (char**)kwlist, &filename))
+    return NULL;
 
   res = self->ptr->load(filename);
 
@@ -119,10 +119,10 @@ py_an_save(py_annoy *self, PyObject *args, PyObject *kwargs) {
   char *filename;
   bool res = false;
   if (!self->ptr) 
-    Py_RETURN_NONE;
-  static char * kwlist[] = {"fn", NULL};
-  if (!PyArg_ParseTupleAndKeywords(args, kwargs, "s", kwlist, &filename))
-    Py_RETURN_NONE;
+    return NULL;
+  static char const * kwlist[] = {"fn", NULL};
+  if (!PyArg_ParseTupleAndKeywords(args, kwargs, "s", (char**)kwlist, &filename))
+    return NULL;
 
   res = self->ptr->save(filename);
 
@@ -154,13 +154,31 @@ get_nns_to_python(const vector<int32_t>& result, const vector<float>& distances,
 }
 
 
+bool check_constraints(py_annoy *self, int32_t item, bool building) {
+  if (item < 0) {
+    PyErr_SetString(PyExc_IndexError, "Item index can not be negative");
+    return false;
+  } else if (!building && item >= self->ptr->get_n_items()) {
+    PyErr_SetString(PyExc_IndexError, "Item index larger than the largest item index");
+    return false;
+  } else {
+    return true;
+  }
+}
+
 static PyObject* 
-py_an_get_nns_by_item(py_annoy *self, PyObject *args) {
+py_an_get_nns_by_item(py_annoy *self, PyObject *args, PyObject *kwargs) {
   int32_t item, n, search_k=-1, include_distances=0;
   if (!self->ptr) 
-    Py_RETURN_NONE;
-  if (!PyArg_ParseTuple(args, "ii|ii", &item, &n, &search_k, &include_distances))
-    Py_RETURN_NONE;
+    return NULL;
+
+  static char const * kwlist[] = {"i", "n", "search_k", "include_distances", NULL};
+  if (!PyArg_ParseTupleAndKeywords(args, kwargs, "ii|ii", (char**)kwlist, &item, &n, &search_k, &include_distances))
+    return NULL;
+
+  if (!check_constraints(self, item, false)) {
+    return NULL;
+  }
 
   vector<int32_t> result;
   vector<float> distances;
@@ -173,19 +191,36 @@ py_an_get_nns_by_item(py_annoy *self, PyObject *args) {
 }
 
 
+bool
+convert_list_to_vector(PyObject* v, int f, vector<float>* w) {
+  if (PyObject_Size(v) != f) {
+    PyErr_SetString(PyExc_IndexError, "Vector has wrong length");
+    return false;
+  }
+  for (int z = 0; z < f; z++) {
+    PyObject *key = PyInt_FromLong(z);
+    PyObject *pf = PyObject_GetItem(v, key);
+    (*w)[z] = PyFloat_AsDouble(pf);
+    Py_DECREF(key);
+    Py_DECREF(pf);
+  }
+  return true;
+}
+
 static PyObject* 
-py_an_get_nns_by_vector(py_annoy *self, PyObject *args) {
+py_an_get_nns_by_vector(py_annoy *self, PyObject *args, PyObject *kwargs) {
   PyObject* v;
   int32_t n, search_k=-1, include_distances=0;
   if (!self->ptr) 
-    Py_RETURN_NONE;
-  if (!PyArg_ParseTuple(args, "Oi|ii", &v, &n, &search_k, &include_distances))
-    Py_RETURN_NONE;
+    return NULL;
+
+  static char const * kwlist[] = {"vector", "n", "search_k", "include_distances", NULL};
+  if (!PyArg_ParseTupleAndKeywords(args, kwargs, "Oi|ii", (char**)kwlist, &v, &n, &search_k, &include_distances))
+    return NULL;
 
   vector<float> w(self->f);
-  for (int z = 0; z < PyList_Size(v) && z < self->f; z++) {
-    PyObject *pf = PyList_GetItem(v,z);
-    w[z] = PyFloat_AsDouble(pf);
+  if (!convert_list_to_vector(v, self->f, &w)) {
+    return NULL;
   }
 
   vector<int32_t> result;
@@ -203,9 +238,13 @@ static PyObject*
 py_an_get_item_vector(py_annoy *self, PyObject *args) {
   int32_t item;
   if (!self->ptr) 
-    Py_RETURN_NONE;
+    return NULL;
   if (!PyArg_ParseTuple(args, "i", &item))
-    Py_RETURN_NONE;
+    return NULL;
+
+  if (!check_constraints(self, item, false)) {
+    return NULL;
+  }
 
   vector<float> v(self->f);
   self->ptr->get_item(item, &v[0]);
@@ -219,18 +258,22 @@ py_an_get_item_vector(py_annoy *self, PyObject *args) {
 
 
 static PyObject* 
-py_an_add_item(py_annoy *self, PyObject *args) {
-  PyObject* l;
+py_an_add_item(py_annoy *self, PyObject *args, PyObject* kwargs) {
+  PyObject* v;
   int32_t item;
   if (!self->ptr) 
-    Py_RETURN_NONE;
-  if (!PyArg_ParseTuple(args, "iO", &item, &l))
-    Py_RETURN_NONE;
+    return NULL;
+  static char const * kwlist[] = {"i", "vector", NULL};
+  if (!PyArg_ParseTupleAndKeywords(args, kwargs, "iO", (char**)kwlist, &item, &v))
+    return NULL;
 
-  vector<float> w(self->f, 0.0);
-  for (int z = 0; z < self->f; z++) {
-    PyObject *pf = PyList_GetItem(l,z);
-    w[z] = PyFloat_AsDouble(pf);
+  if (!check_constraints(self, item, true)) {
+    return NULL;
+  }
+
+  vector<float> w(self->f);
+  if (!convert_list_to_vector(v, self->f, &w)) {
+    return NULL;
   }
   self->ptr->add_item(item, &w[0]);
 
@@ -242,10 +285,10 @@ static PyObject *
 py_an_build(py_annoy *self, PyObject *args, PyObject *kwargs) {
   int q;
   if (!self->ptr) 
-    Py_RETURN_NONE;
-  static char * kwlist[] = {"n_trees", NULL};
-  if (!PyArg_ParseTupleAndKeywords(args, kwargs, "i", kwlist, &q))
-    Py_RETURN_NONE;
+    return NULL;
+  static char const * kwlist[] = {"n_trees", NULL};
+  if (!PyArg_ParseTupleAndKeywords(args, kwargs, "i", (char**)kwlist, &q))
+    return NULL;
 
   Py_BEGIN_ALLOW_THREADS;
   self->ptr->build(q);
@@ -258,7 +301,7 @@ py_an_build(py_annoy *self, PyObject *args, PyObject *kwargs) {
 static PyObject *
 py_an_unbuild(py_annoy *self) {
   if (!self->ptr) 
-    Py_RETURN_NONE;
+    return NULL;
   
   Py_BEGIN_ALLOW_THREADS;
   self->ptr->unbuild();
@@ -271,7 +314,7 @@ py_an_unbuild(py_annoy *self) {
 static PyObject *
 py_an_unload(py_annoy *self) {
   if (!self->ptr) 
-    Py_RETURN_NONE;
+    return NULL;
 
   self->ptr->unload();
 
@@ -283,9 +326,13 @@ static PyObject *
 py_an_get_distance(py_annoy *self, PyObject *args) {
   int32_t i, j;
   if (!self->ptr) 
-    Py_RETURN_NONE;
+    return NULL;
   if (!PyArg_ParseTuple(args, "ii", &i, &j))
-    Py_RETURN_NONE;
+    return NULL;
+
+  if (!check_constraints(self, i, false) || !check_constraints(self, j, false)) {
+    return NULL;
+  }
 
   double d = self->ptr->get_distance(i,j);
   return PyFloat_FromDouble(d);
@@ -295,7 +342,7 @@ py_an_get_distance(py_annoy *self, PyObject *args) {
 static PyObject *
 py_an_get_n_items(py_annoy *self) {
   if (!self->ptr) 
-    Py_RETURN_NONE;
+    return NULL;
 
   int32_t n = self->ptr->get_n_items();
   return PyInt_FromLong(n);
@@ -306,9 +353,9 @@ static PyObject *
 py_an_verbose(py_annoy *self, PyObject *args) {
   int verbose;
   if (!self->ptr) 
-    Py_RETURN_NONE;
+    return NULL;
   if (!PyArg_ParseTuple(args, "i", &verbose))
-    Py_RETURN_NONE;
+    return NULL;
 
   self->ptr->verbose((bool)verbose);
 
@@ -320,9 +367,9 @@ static PyObject *
 py_an_set_seed(py_annoy *self, PyObject *args) {
   int q;
   if (!self->ptr)
-    Py_RETURN_NONE;
+    return NULL;
   if (!PyArg_ParseTuple(args, "i", &q))
-    Py_RETURN_NONE;
+    return NULL;
 
   self->ptr->set_seed(q);
 
@@ -333,14 +380,14 @@ py_an_set_seed(py_annoy *self, PyObject *args) {
 static PyMethodDef AnnoyMethods[] = {
   {"load",	(PyCFunction)py_an_load, METH_VARARGS | METH_KEYWORDS, "Loads (mmaps) an index from disk."},
   {"save",	(PyCFunction)py_an_save, METH_VARARGS | METH_KEYWORDS, "Saves the index to disk."},
-  {"get_nns_by_item",(PyCFunction)py_an_get_nns_by_item, METH_VARARGS, ""},
-  {"get_nns_by_vector",(PyCFunction)py_an_get_nns_by_vector, METH_VARARGS, ""},
-  {"get_item_vector",(PyCFunction)py_an_get_item_vector, METH_VARARGS, ""},
-  {"add_item",(PyCFunction)py_an_add_item, METH_VARARGS, ""},
+  {"get_nns_by_item",(PyCFunction)py_an_get_nns_by_item, METH_VARARGS | METH_KEYWORDS, "Returns the `n` closest items to item `i`.\n\n:param search_k: the query will inspect up to `search_k` nodes.\n`search_k` gives you a run-time tradeoff between better accuracy and speed.\n`search_k` defaults to `n_trees * n` if not provided.\n\n:param include_distances: If `True`, this function will return a\n2 element tuple of lists. The first list contains the `n` closest items.\nThe second list contains the corresponding distances."},
+  {"get_nns_by_vector",(PyCFunction)py_an_get_nns_by_vector, METH_VARARGS | METH_KEYWORDS, "Returns the `n` closest items to vector `vector`.\n\n:param search_k: the query will inspect up to `search_k` nodes.\n`search_k` gives you a run-time tradeoff between better accuracy and speed.\n`search_k` defaults to `n_trees * n` if not provided.\n\n:param include_distances: If `True`, this function will return a\n2 element tuple of lists. The first list contains the `n` closest items.\nThe second list contains the corresponding distances."},
+  {"get_item_vector",(PyCFunction)py_an_get_item_vector, METH_VARARGS, "Returns the vector for item `i` that was previously added."},
+  {"add_item",(PyCFunction)py_an_add_item, METH_VARARGS | METH_KEYWORDS, "Adds item `i` (any nonnegative integer) with vector `v`.\n\nNote that it will allocate memory for `max(i)+1` items."},
   {"build",(PyCFunction)py_an_build, METH_VARARGS | METH_KEYWORDS, "Builds a forest of `n_trees` trees.\n\nMore trees give higher precision when querying. After calling `build`,\nno more items can be added."},
   {"unbuild",(PyCFunction)py_an_unbuild, METH_NOARGS, "Unbuilds the tree in order to allows adding new items.\n\nbuild() has to be called again afterwards in order to\nrun queries."},
   {"unload",(PyCFunction)py_an_unload, METH_NOARGS, "Unloads an index from disk."},
-  {"get_distance",(PyCFunction)py_an_get_distance, METH_VARARGS, ""},
+  {"get_distance",(PyCFunction)py_an_get_distance, METH_VARARGS, "Returns the distance between items `i` and `j`."},
   {"get_n_items",(PyCFunction)py_an_get_n_items, METH_NOARGS, "Returns the number of items in the index."},
   {"verbose",(PyCFunction)py_an_verbose, METH_VARARGS, ""},
   {"set_seed",(PyCFunction)py_an_set_seed, METH_VARARGS, "Sets the seed of Annoy's random number generator."},
